#ifdef __linux__
#define _BSD_SOURCE
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#ifndef USE_NETLIB
#include <net/ethernet.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#else
#include <netlib.h>
#endif

#include "pktlib.h"
#include "pktbuf.h"

pktbuf_t pktbuf_make_udp(pktbuf_t pktbuf, struct header_param *param)
{
  struct udphdr *udphdr;
  int hdrsize, paysize;

  hdrsize = sizeof(*udphdr);
  paysize = pktbuf_get_size(pktbuf);

  udphdr = (struct udphdr *)pktbuf_add_header(pktbuf, hdrsize);
  memset(udphdr, 0, hdrsize);

  udphdr->uh_sport = htons(param->udp.src_port);
  udphdr->uh_dport = htons(param->udp.dst_port);
  udphdr->uh_ulen  = htons(hdrsize + paysize);

  return pktbuf;
}

pktbuf_t pktbuf_make_ip(pktbuf_t pktbuf, struct header_param *param)
{
  struct ip *iphdr;
  int hdrsize, paysize;

  hdrsize = sizeof(struct ip);
  paysize = pktbuf_get_size(pktbuf);

  iphdr = (struct ip *)pktbuf_add_header(pktbuf, hdrsize);
  memset(iphdr, 0, hdrsize);

  iphdr->ip_v   = 4;
  iphdr->ip_hl  = hdrsize >> 2;
  iphdr->ip_len = htons(hdrsize + paysize);
  iphdr->ip_id  = htons(param->ip.id);
  iphdr->ip_ttl = param->ip.ttl;
  iphdr->ip_p   = param->ip.protocol;
  iphdr->ip_src.s_addr = htonl(param->ip.src_ipaddr);
  iphdr->ip_dst.s_addr = htonl(param->ip.dst_ipaddr);

  return pktbuf;
}

pktbuf_t pktbuf_make_ethernet(pktbuf_t pktbuf, struct header_param *param)
{
  struct ether_header *ehdr;

  ehdr = (struct ether_header *)pktbuf_add_header(pktbuf, ETHER_HDR_LEN);

  memcpy(ehdr->ether_dhost, param->ethernet.dst_macaddr, ETHER_ADDR_LEN);
  memcpy(ehdr->ether_shost, param->ethernet.src_macaddr, ETHER_ADDR_LEN);
  ehdr->ether_type = htons(param->ethernet.type);

  return pktbuf;
}
